@@ -1,5 +1,6 @@
 from urllib.parse import urlparse, unquote, urlunparse, quote
 import re
+import typing
 
 
 def connect_to_database(uri: str, library: str=None, for_write: bool=False, **kwargs):
@@ -9,6 +10,7 @@ def connect_to_database(uri: str, library: str=None, for_write: bool=False, **kw
     :param uri:        URI of database.
     :param library:     A specific library to use, if available
     :param for_write:   Whether to enable write operations.
+    :param sniff_only:  True to return a ConnectStrategy instead of a connection.
     :returns:  a DBI driver
     """
     # determine flavor of connection and work out which libraries to try
@@ -36,6 +38,19 @@ def connect_to_database(uri: str, library: str=None, for_write: bool=False, **kw
     raise ValueError(f"No driver installed/supported for {uri_parts[0]}")
 
 
+class ConnectStrategy(object):
+    """
+    How to connect to a database.
+    """
+    def __init__(self, opener: typing.Callable, open_args: dict=None):
+        """
+        :param opener:      The connection method.
+        :param open_args:   Arguments to pass.
+        """
+        self.opener = opener
+        self.open_args = open_args
+
+
 def _process_params(uri: str, **kwargs):
     """
     Use a URI to provide most of the needed values for connecting to a database.  'kwargs' supplies overrides.
@@ -76,20 +91,23 @@ def _process_params(uri: str, **kwargs):
     return params
 
 
-def connect_pymysql(host, port=None, username=None, password=None, database=None, autocommit=True, for_write: bool=False, **kwargs):
+def connect_pymysql(host, port=None, username=None, password=None, database=None, autocommit=True, for_write: bool=False, sniff_only: bool=False, **kwargs):
     try:
         import pymysql
     except ImportError:
         return
     port = port or 3306
-    conn = pymysql.connect(user=username, password=password, host=host, port=port, database=database or None, autocommit=autocommit, **kwargs)
+    params = dict(user=username, password=password, host=host, port=port, database=database or None, autocommit=autocommit, **kwargs)
+    if sniff_only:
+        return ConnectStrategy(opener=pymysql.connect, open_args=params)
+    conn = pymysql.connect(**params)
     if not for_write:
         _set_conn_read_only(conn)
     conn._flavor = "mysql"
     return conn
 
 
-def connect_pymssql(host, port=None, username=None, password=None, database=None, autocommit=True, for_write: bool=False, **kwargs):
+def connect_pymssql(host, port=None, username=None, password=None, database=None, autocommit=True, for_write: bool=False, sniff_only: bool=False, **kwargs):
     try:
         import pymssql
     except ImportError:
@@ -97,7 +115,7 @@ def connect_pymssql(host, port=None, username=None, password=None, database=None
     more = {}
     if "timeout" in kwargs:
         more["timeout"] = int(kwargs["timeout"])
-    conn = pymssql.connect(
+    params = dict(
         server=host,
         user=username,
         password=password,
@@ -105,13 +123,18 @@ def connect_pymssql(host, port=None, username=None, password=None, database=None
         autocommit=autocommit,
         **more
     )
+    if sniff_only:
+        return ConnectStrategy(opener=pymssql.connect, open_args=params)
+    conn = pymssql.connect(
+        **params
+    )
     if not for_write:
         _set_conn_read_only(conn)
     conn._flavor = "mssql"
     return conn
 
 
-def connect_psycopg2(host, port=None, username=None, password=None, database=None, autocommit=True, for_write: bool=False, **kwargs):
+def connect_psycopg2(host, port=None, username=None, password=None, database=None, autocommit=True, for_write: bool=False, sniff_only: bool=False, **kwargs):
     try:
         import psycopg2
         import psycopg2.extensions
@@ -123,7 +146,10 @@ def connect_psycopg2(host, port=None, username=None, password=None, database=Non
     if password:
         dsn_parts["password"] = password
     pg_args_str = " ".join("%s=%s" % (k, v) for k, v in sorted(dsn_parts.items()))
-    conn = psycopg2.connect(pg_args_str, **kwargs)
+    params = dict(dsn=pg_args_str, **kwargs)
+    if sniff_only:
+        return ConnectStrategy(opener=psycopg2.connect, open_args=params)
+    conn = psycopg2.connect(params)
     if autocommit:
         conn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
     if not for_write:
@@ -135,16 +161,20 @@ def connect_psycopg2(host, port=None, username=None, password=None, database=Non
     return conn
 
 
-def connect_sqlite(host, database=None, for_write: bool=False, **kwargs):
+def connect_sqlite(host, database=None, for_write: bool=False, sniff_only: bool=False, **kwargs):
     try:
         import sqlite3
     except ImportError:
         return
     for ignore in ["port", "username", "password", "database", "autocommit"]:
         kwargs.pop(ignore, None)
-    if not for_write:
-        return sqlite3.connect(f'file:{host or database}?mode=ro', uri=True, **kwargs)
-    conn = sqlite3.connect(host, **kwargs)
+    if for_write:
+        params = dict(database=host, **kwargs)
+    else:
+        params = dict(database=f'file:{host or database}?mode=ro', uri=True, **kwargs)
+    if sniff_only:
+        return ConnectStrategy(opener=sqlite3.connect, open_args=params)
+    conn = sqlite3.connect(**params)
     conn._flavor = "sqlite"
     return conn
 
@@ -202,7 +232,7 @@ def db_conn_flavor(connector) -> str:
     if hasattr(connector, "_flavor"):
         return connector._flavor
     name = connector.__class__.__name__.lower()
-    if "postg" in name:
+    if "postg" in name or "psycopg" in name:
         return "postgres"
 
 
