@@ -263,9 +263,13 @@ def sql_enquote_id(s: (str, list, tuple), mode: str=None, always: bool=False, do
     if s == "*" or s.startswith("*["):
         # special cases
         return s
-    if re.match(r'^[a-zA-Z_][a-zA-Z0-9_\-]*$', s) is not None and not always:
+    if (PTN_NOQUOTES_PG if mode == "postgres" else PTN_NOQUOTES).match(s) is not None and not always:
         return s
     # we enquote file-based data sources like MySQL for an obscure reason:
     # - pandas.read_sql() strips double-quotes out of SQL!
     q = '`' if mode in {"mysql", "s3", "abs", "bigquery"} else '"'
     return "%s%s%s" % (q, s.replace(q, q+q), q)
+
+
+PTN_NOQUOTES = re.compile(r'^[a-zA-Z_][a-zA-Z0-9_\-]*$')
+PTN_NOQUOTES_PG = re.compile(r'^[a-z_][a-z0-9_\-]*$')
